==> grunt-cacheinfo-0.4.10.tgz: package/tasks/cacheinfo.js <==
/*
 * Modified @ 2014-12-12
 *
 * Copyright (c) 2014 拔赤
 * Licensed under the MIT license.
 *
 * 生成cache_info.json的脚本，参考这里的实现
 *
 * http://gitlab.alibaba-inc.com/snippets/886
 */
var fs = require('fs');
var path = require('path');
var http = require('http');
var https = require('https');
var url = require('url');
var util = require('util');
var clamUtil = require('clam-util');
var fsExtra = require('fs-extra');
var exec = require('child_process').exec;

module.exports = function (grunt) {
  grunt.registerMultiTask('cacheinfo', 'cacheinfo.', function () {
    // Merge task-specific and/or target-specific options with these defaults.

    /*
     src
     dest
     abc
     */
    var options = this.options({
      // 支持扩展 meta merge 到 abc.json 上
      extMeta: {},
      // weex file dir
      wxDir: 'pages',
      abc: "package.json",
      src: "build_offline",
      dest: "build_offline/cache_info.json",
      genABCJSON: 'build_offline/abc.json',
      distDir: 'build',
      offlineSyncDir: 'offline-trip-sync',
      offlineSyncHost: 'g.alicdn.com'
    });

    var source_files = clamUtil.walk(options.src,
      clamUtil.NORMAL_FILTERS,
      clamUtil.NORMAL_EXFILTERS);
    var all_files = distinct((source_files.css || [])
      .concat(source_files.eot || [])
      .concat(source_files.otf || [])
      .concat(source_files.svg || [])
      .concat(source_files.ttf || [])
      .concat(source_files.woff || [])
      .concat(source_files.html || [])
      .concat(source_files.htm || [])
      .concat(source_files.js || [])
      .concat(source_files.css || [])
      .concat(source_files.png || [])
      .concat(source_files.gif || [])
      .concat(source_files.jpg || [])
      .concat(source_files.swf || []));

    var done = this.async();
    var that = this;
    var pwd = process.cwd();
    var config_url = options.configUrl || 'https://g.alicdn.com/trip/offline-config/index.json';

    if (/http:/.test(config_url)) {
      config_url = config_url.replace(/^http/, 'https');
    }

    getGlobalConfig(function (globalConfig) {

      var pkgConfig = getPkgConfig(globalConfig);
      if (pkgConfig) {
        generatorCacheInfo(pkgConfig, all_files);
      } else {
        done();
      }

    });

    // 根据文件路径得到完整的URL
    function generateFullUrl(config, file) {
      var wap_relative = getWapRelative(config.wapUrl);

      // 从URL解析的nodes
      var wap_nodes = wap_relative.match(/[^\/]+/ig);

      // 从files里解析的nodes
      var source_nodes = file.match(/[^\/]+/ig);

      // 从main中解析的基本路径
      var package_nodes = config.main.match(/[^\/]+/ig);

      var base_nodes = package_nodes;

      // generate_full_url(wap_nodes, package_nodes|base_nodes, file|source_path)

      var temp = 0;
      for (var i = 0; i < source_nodes.length; i++) {
        if (source_nodes[i] != base_nodes[i]) {
          temp = i;
          break;
        }
      }

      var result_nodes = [];
      for (var i = 0; i < wap_nodes.length; i++) {
        if (i == (wap_nodes.length - base_nodes.length) + temp) {
          break;
        } else {
          result_nodes.push(wap_nodes[i]);
        }
      }

      for (var i = 0; i < source_nodes.length; i++) {
        if (i >= temp) {
          result_nodes.push(source_nodes[i]);
        }
      }

      return '/' + result_nodes.join('/');

    }

    function getWapRelative(wapUrl) {
      var wap_relative = wapUrl;
      var parsedUrl = url.parse(wapUrl);
      if (!parsedUrl.host || !parsedUrl.protocol) {
        wapUrl = 'http://' + wapUrl;
        parsedUrl = url.parse(wapUrl);
      }
      wap_relative = parsedUrl.path;
      wap_relative = wap_relative.replace('#/fillinfo', '');
      return wap_relative;
    }

    // => { name:'h5-car',version:'0.2.1',type:'h5',src:'xxxx.zip','wapUrl':'http://h5.m.taobao.com/trip/car/search/index_new.html'}
    function getPkgConfig(globalConfig) {
      var abc = require(path.resolve(process.cwd(), options.abc));
      var pkg = {},
        isPkgFound = false;
      for (var i = 0; i < globalConfig.length; i++) {
        if (globalConfig[i].name == abc.name || (globalConfig[i].name == 'h5-ticket' && abc.name == 'h5-fsticket')) {
          pkg = globalConfig[i];
          isPkgFound = true;
          break;
        }
      }
      // 校验 abc.json 是否缺少必须字段
      var errorTpl = 'abc.json 中缺少 "%s" 字段或为空，请补全后重试！',
        requiredKeys = ['version', 'basePath', 'baseUrl'],
        errorParam = '';

      requiredKeys.forEach(function (key) {
        if (!abc[key]) {
          errorParam = key;
          return;
        }
      });

      if (errorParam) {
        grunt.fail.warn(util.format(errorTpl, errorParam));
        if (isPkgFound) {
          // 降级到总控去找
          grunt.log.writeln('从 ' + config_url + ' 中读取到 ' + abc.name + ' 的配置信息');
          return pkg;
        }
      } else {
        let isAir = (abc.defPublishType === 'weex' || abc.defPublishType === 'webapp') && !abc.awpProxy;
        let wapUrlDomain = options.wapUrlDomain || (isAir ? 'http://market.m.taobao.com/' : 'http://h5.m.taobao.com/');
        let baseUrl = abc.baseUrl.replace(/^\//, '');

        // 迁移至 weex/webapp 兼容老版本
        if (isAir && !/^\/?app\//.test(baseUrl)) {
          // trip/abc => trip/h5-abc
          if (/^h5-/.test(abc.name) && !/^trip\/h5-/.test(baseUrl)) {
            baseUrl = baseUrl.replace(/^trip\//, 'trip/h5-');
          }
          // trip/h5-abc/home => trip/h5-abc/pages/home
          if (baseUrl.indexOf('trip/' + abc.name + '/pages/') !== 0) {
            baseUrl = baseUrl.replace('trip/' + abc.name, 'trip/' + abc.name + '/pages');
          }
          // trip/h5-abc/pages/home => app/trip/h5-abc/pages/home
          baseUrl = 'app/' + baseUrl;
        }

        // 优先取 abc.json 中配置
        return {
          name: abc.name,
          version: abc.version,
          type: 'h5',
          main: abc.basePath,
          wapUrl: wapUrlDomain + baseUrl,
          isAir: isAir
        };
      }
    }

    // 得到 'http://g.alicdn.com/trip/offline-config/index.json'
    function getGlobalConfig(ok) {
      let count = 0;

      function handleError() {
        if (count < 2) {
          count++;
          setTimeout(function() {
            _getGlobalConfig();
          }, 3000);
        } else {
          grunt.fail.fatal('错误:请求不到 ' + config_url);
          done();
        }
      }

      function _getGlobalConfig() {
        https.get(config_url, function (res) {
          var raw = '';
          res.on('data', function (chunk) {
            raw += chunk.toString('utf8');
          });
          res.on('end', function () {
            ok(JSON.parse(raw));
          });
        }).on('error', handleError);
      }

      _getGlobalConfig();
    }

    // 生成 cache_info.json
    function generatorCacheInfo(pkg, all_files) {
      var cache_files = {};
      var weex_cache_files = {};
      var abc = require(path.resolve(process.cwd(), options.abc));
      var packageName = pkg.isAir ? pkg.name : pkg.name.replace(/^h5-/, '');
      var universalWeex = (!abc.universalTargets || abc.universalTargets.weex);
      var awpPathMap = abc.awpPathMap || {};

      all_files.forEach(function (file, k) {
        // ----------这段逻辑是浩志写的，不明用意，我抄过来的
        if (file == pkg.main) {
          cache_files[getWapRelative(pkg.wapUrl)] = file;
        } else {
          var key = generateFullUrl(pkg, file);
          var keyPath = key.replace(/^\/trip\//, '');
          if (awpPathMap[keyPath]) {
            key = '/trip/' + awpPathMap[keyPath];
          }
          cache_files[key] = file;
          if (packageName == 'flight-checkin') {
            key = key.replace(packageName, 'pages');
          } else {
            if (packageName == 'flight-onsale') {
              key = key.replace('flight', 'pages');
              cache_files[key] = file;
              key = key.replace('/flight/onsale', '');
            } else {
              key = key.replace(packageName, 'pages');
            }
          }
          key = key.replace('/pages/pages/', '/pages/');
          if (key) {
            cache_files[key] = file;
          }
        }
        // -----------
      });

      var genABCJSONPath = options.genABCJSON;
      if (genABCJSONPath) {

        var abcJSON = {
          name: abc.name,
          version: abc.version,
          t: Date.now(),
          cache: cache_files
        };

        var extMetaKeys = Object.keys(options.extMeta);
        if (extMetaKeys.length > 0) {
          extMetaKeys.forEach(function (extMetaKey) {
            abcJSON[extMetaKey] = options.extMeta[extMetaKey];
          });
        }

        // 处理 jsbundle map
        if (abc.triprn) {
          abcJSON.triprn = true;
          var pagePrefixArr = Object.keys(cache_files).filter(function (assetPrefix) {
            return /htm(l)?$/.test(assetPrefix);
          });

          abcJSON.tripBundleMap = {};
          pagePrefixArr.forEach(function (pagePrefix) {
            abcJSON.tripBundleMap[pagePrefix] = abc.rnBundle;
          });
          // 合并 abc.json 中的 tripBundleMap
          if (abc.tripBundleMap) {
            Object.keys(abc.tripBundleMap).forEach(function (pagePrefix) {
              abcJSON.tripBundleMap[pagePrefix] = abc.tripBundleMap[pagePrefix];
            });
          }
        }

        // weex 项目映射表生成, awpp 线上路径与本地文件路径映射
        if (abc.type === 'weex') {
          var projectName = abc.name.replace(/^h5-/, '');
          var wxDir = options.wxDir;
          var wxRegExp = new RegExp('^' + wxDir + '\/.*\.js$', 'i');
          all_files.forEach(function (file, k) {
            if (wxRegExp.test(file)) {
              var pathPrefix = '/' + abc.group + '/' + projectName + '/';
              var key = pathPrefix + file;
              var keyPath = key.replace(/^\/trip\//, '').replace(/\/pages\//, '/');
              if (awpPathMap[keyPath]) {
                key = '/trip/' + awpPathMap[keyPath];
              }
              [
                '/js' + key.replace(/\.js$/, '-min.js'),
                '/js' + key,

                // awp 升级支持不需要必须部署在 `/js/` 前缀下
                key.replace(/\.js$/, '-min.js'),
                key
              ].forEach(function (reqPath) {
                abcJSON.cache[reqPath] = file;
              });
            }
          });
        }

        if (pkg.isAir) {
          if (abc.defPublishType === 'weex') {
            Object.keys(cache_files).forEach(function(p) {
              const f = cache_files[p];
              // 处理 html
              if (/\.html$/.test(f)) {
                if (/\/index\.html$/.test(f)) {
                  cache_files[p.replace(/\/index\.html/, abc.composite ? '.html' : '')] = f;
                }
              }
              // 处理 index.weex.js
              if (/\.js$/.test(f) && universalWeex) {
                if (/pages\/[^/]+\/index\.(weex|native)(-min)?\.js$/.test(f)) {
                  weex_cache_files[p.replace(/\/index\.(weex|native)(-min)?\.js$/, abc.composite ? '.html' : '')] = f;
                } else {
                  weex_cache_files[p] = cache_files[p];
                }
              }
            });

            if (Object.keys(weex_cache_files).length) {
              abcJSON.weexCache = weex_cache_files;
            }
          }
          abcJSON.cacheInfoVersion = '1.0.0';
        }

        // 备份目录
        fsExtra.copySync(options.src, path.join(options.distDir, options.offlineSyncDir));

        weex_cache_files.onlinePrefix = cache_files.onlinePrefix = `https:\/\/${options.offlineSyncHost}\/` + abc.group + '\/' + abc.name + '\/' + abc.version + '\/' + options.offlineSyncDir + '\/';

        clamUtil.fs.writeJSONFile(genABCJSONPath, abcJSON, function (err) {
          if (err) {
            console.log(err);
          } else {
            console.log(">>> write to " + genABCJSONPath);
          }
          teardown(genABCJSONPath);
          done();
        });

      } else {

        clamUtil.fs.writeJSONFile(options.dest, cache_files, function (err) {
          if (err) {
            console.log(err);
          } else {
            console.log(">>> update " + options.dest);
          }
          teardown(options.dest);
          done();
        });
      }
    }

    function teardown(filePath) {
      var bf = fs.readFileSync(filePath);
      var txt = bf.toString('utf8');
      fs.writeFileSync(filePath, txt.replace(/\//igm, '\\/'));
    }

    function distinct(A) {
      if (!(A instanceof Array) || A.length <= 1)return A;
      var a = [], b = [];
      for (var i = 1; i < A.length; i++) {
        for (var j = 0; j < i; j++) {
          if (inArray(j, b))continue;
          if (A[j] == A[i]) {
            b.push(j);
          }
        }
      }
      for (var i = 0; i < A.length; i++) {
        if (inArray(i, b))continue;
        a.push(A[i]);
      }
      return a;
    }

    function inArray(v, a) {
      var o = false;
      for (var i = 0, m = a.length; i < m; i++) {
        if (a[i] == v) {
          o = true;
          break;
        }
      }
      return o;
    }

  });

};

==> grunt-cacheinfo-0.4.12.tgz: package/tasks/cacheinfo.js <==
/*
 * Modified @ 2014-12-12
 *
 * Copyright (c) 2014 拔赤
 * Licensed under the MIT license.
 *
 * 生成cache_info.json的脚本，参考这里的实现
 *
 * http://gitlab.alibaba-inc.com/snippets/886
 */
var fs = require('fs');
var path = require('path');
var http = require('http');
var https = require('https');
var url = require('url');
var util = require('util');
var clamUtil = require('clam-util');
var fsExtra = require('fs-extra');
var exec = require('child_process').exec;

module.exports = function (grunt) {
  grunt.registerMultiTask('cacheinfo', 'cacheinfo.', function () {
    // Merge task-specific and/or target-specific options with these defaults.

    /*
     src
     dest
     abc
     */
    var options = this.options({
      // 支持扩展 meta merge 到 abc.json 上
      extMeta: {},
      // weex file dir
      wxDir: 'pages',
      abc: "package.json",
      src: "build_offline",
      dest: "build_offline/cache_info.json",
      genABCJSON: 'build_offline/abc.json',
      distDir: 'build',
      offlineSyncDir: 'offline-trip-sync',
      offlineSyncHost: 'g.alicdn.com'
    });

    var source_files = clamUtil.walk(options.src,
      clamUtil.NORMAL_FILTERS,
      clamUtil.NORMAL_EXFILTERS);
    var all_files = distinct((source_files.css || [])
      .concat(source_files.eot || [])
      .concat(source_files.otf || [])
      .concat(source_files.svg || [])
      .concat(source_files.ttf || [])
      .concat(source_files.woff || [])
      .concat(source_files.html || [])
      .concat(source_files.htm || [])
      .concat(source_files.js || [])
      .concat(source_files.css || [])
      .concat(source_files.png || [])
      .concat(source_files.gif || [])
      .concat(source_files.jpg || [])
      .concat(source_files.swf || []));

    var done = this.async();
    var that = this;
    var pwd = process.cwd();
    var config_url = options.configUrl || 'https://g.alicdn.com/trip/offline-config/index.json';

    if (/http:/.test(config_url)) {
      config_url = config_url.replace(/^http/, 'https');
    }

    getGlobalConfig(function (globalConfig) {

      var pkgConfig = getPkgConfig(globalConfig);
      if (pkgConfig) {
        generatorCacheInfo(pkgConfig, all_files);
      } else {
        done();
      }

    });

    // 根据文件路径得到完整的URL
    function generateFullUrl(config, file) {
      var wap_relative = getWapRelative(config.wapUrl);

      // 从URL解析的nodes
      var wap_nodes = wap_relative.match(/[^\/]+/ig);

      // 从files里解析的nodes
      var source_nodes = file.match(/[^\/]+/ig);

      // 从main中解析的基本路径
      var package_nodes = config.main.match(/[^\/]+/ig);

      var base_nodes = package_nodes;

      // generate_full_url(wap_nodes, package_nodes|base_nodes, file|source_path)

      var temp = 0;
      for (var i = 0; i < source_nodes.length; i++) {
        if (source_nodes[i] != base_nodes[i]) {
          temp = i;
          break;
        }
      }

      var result_nodes = [];
      for (var i = 0; i < wap_nodes.length; i++) {
        if (i == (wap_nodes.length - base_nodes.length) + temp) {
          break;
        } else {
          result_nodes.push(wap_nodes[i]);
        }
      }

      for (var i = 0; i < source_nodes.length; i++) {
        if (i >= temp) {
          result_nodes.push(source_nodes[i]);
        }
      }

      return '/' + result_nodes.join('/');

    }

    function getWapRelative(wapUrl) {
      var wap_relative = wapUrl;
      var parsedUrl = url.parse(wapUrl);
      if (!parsedUrl.host || !parsedUrl.protocol) {
        wapUrl = 'http://' + wapUrl;
        parsedUrl = url.parse(wapUrl);
      }
      wap_relative = parsedUrl.path;
      wap_relative = wap_relative.replace('#/fillinfo', '');
      return wap_relative;
    }

    // => { name:'h5-car',version:'0.2.1',type:'h5',src:'xxxx.zip','wapUrl':'http://h5.m.taobao.com/trip/car/search/index_new.html'}
    function getPkgConfig(globalConfig) {
      var abc = require(path.resolve(process.cwd(), options.abc));
      var pkg = {},
        isPkgFound = false;
      for (var i = 0; i < globalConfig.length; i++) {
        if (globalConfig[i].name == abc.name || (globalConfig[i].name == 'h5-ticket' && abc.name == 'h5-fsticket')) {
          pkg = globalConfig[i];
          isPkgFound = true;
          break;
        }
      }
      // 校验 abc.json 是否缺少必须字段
      var errorTpl = 'abc.json 中缺少 "%s" 字段或为空，请补全后重试！',
        requiredKeys = ['version', 'basePath', 'baseUrl'],
        errorParam = '';

      requiredKeys.forEach(function (key) {
        if (!abc[key]) {
          errorParam = key;
          return;
        }
      });

      if (errorParam) {
        grunt.fail.warn(util.format(errorTpl, errorParam));
        if (isPkgFound) {
          // 降级到总控去找
          grunt.log.writeln('从 ' + config_url + ' 中读取到 ' + abc.name + ' 的配置信息');
          return pkg;
        }
      } else {
        let isFlugy = !!abc.flugy;
        let group = (abc.group || 'trip') + '/';
        let isAir = (abc.defPublishType === 'weex' || abc.defPublishType === 'webapp') && !abc.awpProxy;
        let wapUrlDomain = options.wapUrlDomain || (isAir ? 'http://market.m.taobao.com/' : 'http://h5.m.taobao.com/');
        let baseUrl = abc.baseUrl.replace(/^\//, '');

        // 迁移至 weex/webapp 兼容老版本
        if (isAir && !/^\/?app\//.test(baseUrl)) {
          // trip/abc => trip/h5-abc
          if (/^h5-/.test(abc.name) && !/^trip\/h5-/.test(baseUrl)) {
            baseUrl = baseUrl.replace(/^trip\//, 'trip/h5-');
          }
          // trip/h5-abc/home => trip/h5-abc/pages/home
          // rxplugin/xxx/home => rxplugin/xxx/pages/home
          if (baseUrl.indexOf(group + abc.name + '/pages/') !== 0) {
            baseUrl = baseUrl.replace(group + abc.name, group + abc.name + '/pages');
          }
          // trip/h5-abc/pages/home => app/trip/h5-abc/pages/home
          baseUrl = 'app/' + baseUrl;
        }

        // 优先取 abc.json 中配置
        return {
          name: abc.name,
          version: abc.version,
          type: 'h5',
          main: abc.basePath,
          wapUrl: wapUrlDomain + baseUrl,
          isAir: isAir,
          isFlugy: isFlugy
        };
      }
    }

    // 得到 'http://g.alicdn.com/trip/offline-config/index.json'
    function getGlobalConfig(ok) {
      let count = 0;

      function handleError() {
        if (count < 2) {
          count++;
          setTimeout(function() {
            _getGlobalConfig();
          }, 3000);
        } else {
          grunt.fail.fatal('错误:请求不到 ' + config_url);
          done();
        }
      }

      function _getGlobalConfig() {
        https.get(config_url, function (res) {
          var raw = '';
          res.on('data', function (chunk) {
            raw += chunk.toString('utf8');
          });
          res.on('end', function () {
            ok(JSON.parse(raw));
          });
        }).on('error', handleError);
      }

      _getGlobalConfig();
    }

    // 生成 cache_info.json
    function generatorCacheInfo(pkg, all_files) {
      var cache_files = {};
      var weex_cache_files = {};
      var flugy_cache_files = {};
      var abc = require(path.resolve(process.cwd(), options.abc));
      var packageName = pkg.isAir ? pkg.name : pkg.name.replace(/^h5-/, '');
      var universalWeex = (!abc.universalTargets || abc.universalTargets.weex);
      var awpPathMap = abc.awpPathMap || {};

      all_files.forEach(function (file, k) {
        // ----------这段逻辑是浩志写的，不明用意，我抄过来的
        if (file == pkg.main) {
          cache_files[getWapRelative(pkg.wapUrl)] = file;
        } else {
          var key = generateFullUrl(pkg, file);
          var keyPath = key.replace(/^\/trip\//, '');
          if (awpPathMap[keyPath]) {
            key = '/trip/' + awpPathMap[keyPath];
          }
          cache_files[key] = file;
          if (packageName == 'flight-checkin') {
            key = key.replace(packageName, 'pages');
          } else {
            if (packageName == 'flight-onsale') {
              key = key.replace('flight', 'pages');
              cache_files[key] = file;
              key = key.replace('/flight/onsale', '');
            } else {
              key = key.replace(packageName, 'pages');
            }
          }
          key = key.replace('/pages/pages/', '/pages/');
          if (key) {
            cache_files[key] = file;
          }
        }
        // -----------
      });

      var genABCJSONPath = options.genABCJSON;
      if (genABCJSONPath) {

        var abcJSON = {
          name: abc.name,
          version: abc.version,
          t: Date.now(),
          cache: cache_files
        };

        var extMetaKeys = Object.keys(options.extMeta);
        if (extMetaKeys.length > 0) {
          extMetaKeys.forEach(function (extMetaKey) {
            abcJSON[extMetaKey] = options.extMeta[extMetaKey];
          });
        }

        // 处理 jsbundle map
        if (abc.triprn) {
          abcJSON.triprn = true;
          var pagePrefixArr = Object.keys(cache_files).filter(function (assetPrefix) {
            return /htm(l)?$/.test(assetPrefix);
          });

          abcJSON.tripBundleMap = {};
          pagePrefixArr.forEach(function (pagePrefix) {
            abcJSON.tripBundleMap[pagePrefix] = abc.rnBundle;
          });
          // 合并 abc.json 中的 tripBundleMap
          if (abc.tripBundleMap) {
            Object.keys(abc.tripBundleMap).forEach(function (pagePrefix) {
              abcJSON.tripBundleMap[pagePrefix] = abc.tripBundleMap[pagePrefix];
            });
          }
        }

        // weex 项目映射表生成, awpp 线上路径与本地文件路径映射
        if (abc.type === 'weex') {
          var projectName = abc.name.replace(/^h5-/, '');
          var wxDir = options.wxDir;
          var wxRegExp = new RegExp('^' + wxDir + '\/.*\.js$', 'i');
          all_files.forEach(function (file, k) {
            if (wxRegExp.test(file)) {
              var pathPrefix = '/' + abc.group + '/' + projectName + '/';
              var key = pathPrefix + file;
              var keyPath = key.replace(/^\/trip\//, '').replace(/\/pages\//, '/');
              if (awpPathMap[keyPath]) {
                key = '/trip/' + awpPathMap[keyPath];
              }
              [
                '/js' + key.replace(/\.js$/, '-min.js'),
                '/js' + key,

                // awp 升级支持不需要必须部署在 `/js/` 前缀下
                key.replace(/\.js$/, '-min.js'),
                key
              ].forEach(function (reqPath) {
                abcJSON.cache[reqPath] = file;
              });
            }
          });
        }

        if (pkg.isAir) {
          if (abc.defPublishType === 'weex') {
            Object.keys(cache_files).forEach(function(p) {
              const f = cache_files[p];
              // 处理 html
              if (/\.html$/.test(f)) {
                if (/\/index\.html$/.test(f)) {
                  cache_files[p.replace(/\/index\.html/, abc.composite ? '.html' : '')] = f;
                }
              }
              // 处理 index.weex.js
              if (/\.js$/.test(f) && universalWeex) {
                if (/pages\/[^/]+\/index\.(weex|native)(-min)?\.js$/.test(f)) {
                  weex_cache_files[p.replace(/\/index\.(weex|native)(-min)?\.js$/, abc.composite ? '.html' : '')] = f;
                } else {
                  weex_cache_files[p] = cache_files[p];
                }
              }
            });

            if (Object.keys(weex_cache_files).length) {
              abcJSON.weexCache = weex_cache_files;
            }
          }
          abcJSON.cacheInfoVersion = '1.0.0';
        }

        if (pkg.isFlugy) {
          var postFixes = ['.web.js', '.entry.js', '-min.js', '.min.js', '.js'];
          Object.keys(cache_files).forEach(function(p) {
            if (
              /\.html$/.test(p) ||
              !/\./.test(p)
            ) {
              // weex/rax: index.web.js
              // react: index.entry.js
              // other: index-min.js/index.min.js/index.js
              postFixes.some(function(s) {
                var ff = cache_files[p].replace(/\.html/, s);
                if (fs.existsSync(path.join(options.src, ff))) {
                  flugy_cache_files[p] = ff;
                  return true;
                }
              });
            }
          });

          if (Object.keys(flugy_cache_files).length) {
            abcJSON.flugyCache = flugy_cache_files;
            abcJSON.cacheInfoVersion = '1.0.0';
          }
        }

        // 备份目录
        fsExtra.copySync(options.src, path.join(options.distDir, options.offlineSyncDir));

        cache_files.onlinePrefix = `https:\/\/${options.offlineSyncHost}\/` + abc.group + '\/' + abc.name + '\/' + abc.version + '\/' + options.offlineSyncDir + '\/';

        clamUtil.fs.writeJSONFile(genABCJSONPath, abcJSON, function (err) {
          if (err) {
            console.log(err);
          } else {
            console.log(">>> write to " + genABCJSONPath);
          }
          teardown(genABCJSONPath);
          done();
        });

      } else {

        clamUtil.fs.writeJSONFile(options.dest, cache_files, function (err) {
          if (err) {
            console.log(err);
          } else {
            console.log(">>> update " + options.dest);
          }
          teardown(options.dest);
          done();
        });
      }
    }

    function teardown(filePath) {
      var bf = fs.readFileSync(filePath);
      var txt = bf.toString('utf8');
      fs.writeFileSync(filePath, txt.replace(/\//igm, '\\/'));
    }

    function distinct(A) {
      if (!(A instanceof Array) || A.length <= 1)return A;
      var a = [], b = [];
      for (var i = 1; i < A.length; i++) {
        for (var j = 0; j < i; j++) {
          if (inArray(j, b))continue;
          if (A[j] == A[i]) {
            b.push(j);
          }
        }
      }
      for (var i = 0; i < A.length; i++) {
        if (inArray(i, b))continue;
        a.push(A[i]);
      }
      return a;
    }

    function inArray(v, a) {
      var o = false;
      for (var i = 0, m = a.length; i < m; i++) {
        if (a[i] == v) {
          o = true;
          break;
        }
      }
      return o;
    }

  });

};
